@@ -25,24 +25,30 @@
 
 
 /*
-* Genome.java
-*
-* Created on November 9, 2007, 9:05 AM
-*
-* To change this template, choose Tools | Template Manager
-* and open the template in the editor.
-*/
+ * Genome.java
+ *
+ * Created on November 9, 2007, 9:05 AM
+ *
+ * To change this template, choose Tools | Template Manager
+ * and open the template in the editor.
+ */
 package org.broad.igv.feature.genome;
 
-import org.apache.log4j.Logger;
+import org.apache.commons.math3.stat.StatUtils;
 import org.broad.igv.Globals;
 import org.broad.igv.feature.Chromosome;
 import org.broad.igv.feature.Cytoband;
+import org.broad.igv.logging.LogManager;
+import org.broad.igv.logging.Logger;
 import org.broad.igv.track.FeatureTrack;
+import org.broad.igv.track.Track;
 import org.broad.igv.ui.panel.ReferenceFrame;
 import org.broad.igv.util.ResourceLocator;
 
-import java.io.*;
+import java.io.BufferedReader;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.InputStreamReader;
 import java.util.*;
 
 /**
@@ -51,13 +57,15 @@ import java.util.*;
  */
 public class Genome {
 
-    private static Logger log = Logger.getLogger(Genome.class);
-    public static final int MAX_WHOLE_GENOME = 10000;
+    private static Logger log = LogManager.getLogger(Genome.class);
+    public static final int MAX_WHOLE_GENOME_LONG = 100;
+
+    private static Object aliasLock = new Object();
 
     private String id;
     private String displayName;
     private List<String> chromosomeNames;
-    private ArrayList<String> longChromosomeNames;
+    private List<String> longChromosomeNames;
     private LinkedHashMap<String, Chromosome> chromosomeMap;
     private long totalLength = -1;
     private long nominalLength = -1;
@@ -67,15 +75,10 @@ public class Genome {
     private FeatureTrack geneTrack;
     private String species;
     private String ucscID;
-    private GenomeDescriptor descriptor;   // Can be null
-    private ArrayList<ResourceLocator> annotationResources;
-
-
-    public Genome(String id, String displayName, Sequence sequence, boolean chromosOrdered, GenomeDescriptor descriptor) {
-        this(id, displayName, sequence, chromosOrdered);
-        this.descriptor = descriptor;
-
-    }
+    private String blatDB;
+    private List<ResourceLocator> annotationResources;
+    private Map<ResourceLocator, List<Track>> annotationTracks;
+    private boolean showWholeGenomeView = true;
 
     /**
      * @param id
@@ -84,35 +87,33 @@ public class Genome {
      * @param chromosOrdered Whether the chromosomes are already ordered. If false, they will be sorted.
      */
     public Genome(String id, String displayName, Sequence sequence, boolean chromosOrdered) {
+
         this.id = id;
         this.displayName = displayName;
-        this.chrAliasTable = new HashMap<String, String>();
-        this.sequence = sequence;
-        chromosomeNames = sequence.getChromosomeNames();
+        this.chrAliasTable = new HashMap<>();
+        this.sequence = (sequence instanceof InMemorySequence) ? sequence : new SequenceWrapper(sequence);
+        this.chromosomeNames = sequence.getChromosomeNames();
         this.ucscID = ucsdIDMap.containsKey(id) ? ucsdIDMap.get(id) : id;
+        this.chromosomeMap = new LinkedHashMap<>();
 
-        List<Chromosome> tmpChromos = new ArrayList<Chromosome>(chromosomeNames.size());
         int maxLength = -1;
-        chromosomeMap = new LinkedHashMap<String, Chromosome>(tmpChromos.size());
-
         for (int i = 0; i < chromosomeNames.size(); i++) {
             String chr = chromosomeNames.get(i);
             int length = sequence.getChromosomeLength(chr);
-            maxLength = Math.max(maxLength, length);
-            Chromosome chromo = new Chromosome(i, chr, length);
-            tmpChromos.add(chromo);
-
-            if (chromosOrdered) {
-                chromosomeMap.put(chr, chromo);
-            }
+            maxLength = length > maxLength ? length : maxLength;
+            chromosomeMap.put(chr, new Chromosome(i, chr, length));
         }
 
         if (!chromosOrdered) {
-            ChromosomeComparator.sortChromosomeList(tmpChromos, maxLength / 10, chromosomeMap);
-            chromosomeNames = new ArrayList<String>(chromosomeMap.keySet());
+            List<Chromosome> chromosomeList = new ArrayList<>(chromosomeMap.values());
+            Collections.sort(chromosomeList, new ChromosomeComparator(maxLength / 10));
+            for (int ii = 0; ii < chromosomeList.size(); ii++) {
+                Chromosome chrom = chromosomeList.get(ii);
+                chrom.setIndex(ii);
+                chromosomeNames.set(ii, chrom.getName());
+            }
         }
 
-
         initializeChromosomeAliases();
     }
 
@@ -143,7 +144,7 @@ public class Genome {
     public String getCanonicalChrName(String str) {
         if (str == null) {
             return str;
-        } else if (chrAliasTable.containsKey(str)){
+        } else if (chrAliasTable.containsKey(str)) {
             return chrAliasTable.get(str);
         } else {
             return str;
@@ -154,15 +155,6 @@ public class Genome {
         return chrAliasTable.containsKey(str);
     }
 
-    public GenomeDescriptor getDescriptor() {
-        return descriptor;
-    }
-
-    public Map<String, String> getChrAliasTable() {
-        return chrAliasTable;
-    }
-
-
     /**
      * Populate the chr alias table.  The input is a collection of chromosome synonym lists.  The
      * directionality is determined by the "true" chromosome names.
@@ -171,7 +163,8 @@ public class Genome {
      */
     public void addChrAliases(Collection<Collection<String>> synonymsList) {
 
-        if (chrAliasTable == null) chrAliasTable = new HashMap<String, String>();
+        if (synonymsList == null) return;
+        if (chrAliasTable == null) chrAliasTable = new HashMap<>();
 
         // Convert names to a set for fast "contains" testing.
         Set<String> chrNameSet = new HashSet<String>(chromosomeNames);
@@ -240,7 +233,7 @@ public class Genome {
             } else {
                 autoAliases.put("chr" + name, name);
             }
-            if(count++ == 50) break;
+            if (count++ == 50) break;
         }
 
         // Special case for human and mouse -- for other genomes define these in the alias file.
@@ -293,7 +286,7 @@ public class Genome {
      * @return
      */
     public String getHomeChromosome() {
-        if (chromosomeNames.size() == 1 || chromosomeNames.size() > MAX_WHOLE_GENOME) {
+        if (showWholeGenomeView == false || chromosomeNames.size() == 1 || getLongChromosomeNames().size() > MAX_WHOLE_GENOME_LONG) {
             return chromosomeNames.get(0);
         } else {
             return Globals.CHR_ALL;
@@ -386,6 +379,18 @@ public class Genome {
         return id;
     }
 
+    public String getBlatDB() {
+        return blatDB != null ? blatDB : id;
+    }
+
+    public void setBlatDB(String blatDB) {
+        this.blatDB = blatDB;
+    }
+
+    public void setUcscID(String ucscID) {
+        this.ucscID = ucscID;
+    }
+
     public String getUCSCId() {
         return ucscID == null ? id : ucscID;
     }
@@ -429,7 +434,7 @@ public class Genome {
      */
 
     public byte[] getSequence(String chr, int start, int end) {
-        return getSequence(chr, start, end, true);
+        return getSequence(getCanonicalChrName(chr), start, end, true);
     }
 
     public byte[] getSequence(String chr, int start, int end, boolean useCache) {
@@ -493,7 +498,7 @@ public class Genome {
     }
 
     /**
-     * Return the annotation track associated with this genome.   Can return null
+     * Return the annotation track associated with this genome.  This is a legacy ".genome" artifact, can return null.
      *
      * @return a FeatureTrack, or null
      */
@@ -502,44 +507,64 @@ public class Genome {
     }
 
     /**
-     * Return "getChromosomeNames()" with small chromosomes removed.
+     * Return names of "long" chromosomes relative to a fraction of the median length.  The intent here is to
+     * remove small contigs in an otherwise well assembled genome to support a "whole genome" view.  We first sort
+     * chromosomes in length order, then look for the first large break in size.
      *
      * @return
      */
     public List<String> getLongChromosomeNames() {
-        if (longChromosomeNames == null) {
-            longChromosomeNames = new ArrayList<String>(getAllChromosomeNames().size());
-            long genomeLength = getTotalLength();
-            int maxChromoLength = -1;
-            for (String chrName : getAllChromosomeNames()) {
-                Chromosome chr = getChromosome(chrName);
-                int length = chr.getLength();
-                maxChromoLength = Math.max(maxChromoLength, length);
-                if (length > (genomeLength / 3000)) {
-                    longChromosomeNames.add(chrName);
-                }
-            }
 
-            /**
-             * At this point, we should have some long chromosome names.
-             * However, some genomes (draft ones perhaps) maybe have many small ones
-             * which aren't big enough. We arbitrarily take those which are above
-             * half the size of the max, only if the first method didn't work.
-             */
-            if (longChromosomeNames.size() == 0) {
-                for (String chrName : getAllChromosomeNames()) {
-                    Chromosome chr = getChromosome(chrName);
-                    int length = chr.getLength();
-                    if (length > maxChromoLength / 2) {
-                        longChromosomeNames.add(chrName);
+        if (longChromosomeNames == null) {
+            longChromosomeNames = new ArrayList<>();
+            if (chromosomeMap.size() < 100) {
+                // Keep all chromosomes within 10% of the mean in length
+                double[] lengths = new double[chromosomeMap.size()];
+                int idx = 0;
+                for (Chromosome c : chromosomeMap.values()) {
+                    lengths[idx++] = c.getLength();
+                }
+                double mean = StatUtils.mean(lengths);
+                double std = Math.sqrt(StatUtils.variance(lengths));
+                double min = 0.1 * mean;
+                for (String chr : getAllChromosomeNames()) {
+                    if (chromosomeMap.get(chr).getLength() > min) {
+                        longChromosomeNames.add(chr);
+                    }
+                }
+            } else {
+                // Long list, likely many small contigs.  Search for a break between long (presumably assembled)
+                // chromosomes and small contigs.
+                List<Chromosome> allChromosomes = new ArrayList<>(chromosomeMap.values());
+                allChromosomes.sort((c1, c2) -> c2.getLength() - c1.getLength());
+
+                Chromosome lastChromosome = null;
+                Set<String> tmp = new HashSet<>();
+                for (Chromosome c : allChromosomes) {
+                    if (lastChromosome != null) {
+                        double delta = lastChromosome.getLength() - c.getLength();
+                        if (delta / lastChromosome.getLength() > 0.7) {
+                            break;
+                        }
+                    }
+                    tmp.add(c.getName());
+                    lastChromosome = c;
+                }
+
+
+                for (String chr : getAllChromosomeNames()) {
+                    if (tmp.contains(chr)) {
+                        longChromosomeNames.add(chr);
                     }
                 }
             }
         }
         return longChromosomeNames;
+
     }
 
     public long getNominalLength() {
+
         if (nominalLength < 0) {
             nominalLength = 0;
             for (String chrName : getLongChromosomeNames()) {
@@ -551,12 +576,13 @@ public class Genome {
     }
 
 
-    // TODO A hack (obviously),  we need to record a species in the genome definitions
+    // TODO A hack (obviously),  we need to record a species in the genome definitions to support old style
+    // blat servers.
     private static Map<String, String> ucscSpeciesMap;
 
     private static synchronized String getSpeciesForID(String id) {
         if (ucscSpeciesMap == null) {
-            ucscSpeciesMap = new HashMap<String, String>();
+            ucscSpeciesMap = new HashMap<>();
 
             InputStream is = null;
 
@@ -609,11 +635,43 @@ public class Genome {
         return sequence.isLoaded(frame);
     }
 
-    public void setAnnotationResources(ArrayList<ResourceLocator> annotationResources) {
+    public void setAnnotationResources(List<ResourceLocator> annotationResources) {
         this.annotationResources = annotationResources;
     }
 
-    public ArrayList<ResourceLocator> getAnnotationResources() {
+    public List<ResourceLocator> getAnnotationResources() {
         return annotationResources;
     }
+
+    public Map<ResourceLocator, List<Track>> getAnnotationTracks() {
+        return annotationTracks;
+    }
+
+    public void setAnnotationTracks(Map<ResourceLocator, List<Track>> annotationTracks) {
+        this.annotationTracks = annotationTracks;
+    }
+
+    /**
+     * Mock genome for unit tests
+     */
+
+    private Genome(String id) {
+        this.id = id;
+    }
+
+    ;
+
+    public static Genome mockGenome = new Genome("hg19");
+
+    public void setShowWholeGenomeView(boolean showWholeGenomeView) {
+        this.showWholeGenomeView = showWholeGenomeView;
+    }
+
+    public boolean getShowWholeGenomeView() {
+        return showWholeGenomeView;
+    }
+
+    public void setLongChromosomeNames(List<String> chrNames) {
+        this.longChromosomeNames = chrNames;
+    }
 }
