@@ -57,13 +57,19 @@ final public class Constants {
     public static final String CRAM_CACHE_DIRECTORY = "CRAM.CACHE_DIRECTORY";
     public static final String CRAM_CACHE_SIZE = "CRAM.CACHE_SIZE";
 
-    // Search ("go to") options
+    // Search ("go to") and next feature ("F" and "B") options
     public static final String SEARCH_ZOOM = "SEARCH_ZOOM";
     public static final String FLANKING_REGION = "FLANKING_REGION";
 
+    public static final String NEXT_FIT_TO_WINDOW = "NEXT_FIT_TO_WINDOW";
+
+    public static final String NEXT_FLANKING_REGION = "NEXT_FLANKING_REGION";
+
     // Generic track options
     public static final String BYPASS_FILE_AUTO_DISCOVERY = "BYPASS_FILE_AUTO_DISCOVERY";
     public static final String TRACK_ATTRIBUTE_NAME_KEY = "TRACK_ATTRIBUTE_NAME_KEY";
+
+    public static final String FEATURE_NAME_PROPERTY = "FEATURE_NAME_PROPERTY";
     public static final String INITIAL_TRACK_HEIGHT = "15";
     public static final String COLOR_SCALE_KEY = "COLOR_SCALE_";
     public static final String TRACK_HEIGHT_KEY = "IGV.track.height";
@@ -98,6 +104,9 @@ final public class Constants {
     public static final String SAM_ALLELE_THRESHOLD = "SAM.ALLELE_THRESHOLD";
     public static final String SAM_ALLELE_USE_QUALITY = "SAM.ALLELE_USE_QUALITY";
     public static final String SAM_QUALITY_THRESHOLD = "SAM.QUALITY_THRESHOLD";
+    public static final String SAM_SHADE_QUALITY_LOW = "SAM.SHADE_QUALITY_LOW";
+    public static final String SAM_SHADE_QUALITY_HIGH = "SAM.SHADE_QUALITY_HIGH";
+    public static final String SAM_SHADE_ALIGNMENT_BY = "SAM.SHADE_ALIGNMENT_BY";
     public static final String SAM_ALIGNMENT_SCORE_THRESHOLD = "SAM.ALIGNMENT_SCORE_THRESHOLD";
     public static final String SAM_BASEMOD_THRESHOLD = "SAM.BASEMOD_THRESHOLD";
     public static final String SAM_COMPUTE_ISIZES = "SAM.COMPUTE_ISIZES";
@@ -162,6 +171,7 @@ final public class Constants {
     public static final String SAM_COLOR_G = "SAM.COLOR.G";
     public static final String SAM_COLOR_N = "SAM.COLOR.N";
     public static final String SAM_DISPLAY_MODE = "SAM.DISPLAY_MODE";
+    public static final String SAM_DISPLAY_PAIRED = "SAM.DISPLAY_PAIRED";
     public static final String KNOWN_SNPS = "KNOWN_SNPS_FILE";
 
     // Sequence track settings
@@ -304,7 +314,10 @@ final public class Constants {
             SAM_LARGE_INDELS_THRESHOLD,
             SAM_SHOW_INSERTION_MARKERS,
             SAM_GROUP_OPTION,
-            SAM_GROUP_BY_TAG
+            SAM_GROUP_BY_TAG,
+            SAM_SHADE_QUALITY_LOW,
+            SAM_SHADE_QUALITY_HIGH,
+            SAM_SHADE_ALIGNMENT_BY
     );
 
     /**
