@@ -75,7 +75,7 @@ public abstract class AbstractTrack implements Track {
 
     private String attributeKey;
     private String name;
-    private String url;
+    private String featureInfoURL;
     private boolean itemRGB = true;
 
     private boolean useScore;
@@ -90,8 +90,6 @@ public abstract class AbstractTrack implements Track {
 
     private int top;
     protected int minimumHeight = -1;
-    protected int maximumHeight = 1000;
-
     private TrackType trackType = TrackType.OTHER;
 
     private boolean selected = false;
@@ -152,12 +150,12 @@ public abstract class AbstractTrack implements Track {
         // Ignore by default
     }
 
-    public String getUrl() {
-        return url;
+    public String getFeatureInfoURL() {
+        return featureInfoURL;
     }
 
-    public void setUrl(String url) {
-        this.url = url;
+    public void setFeatureInfoURL(String featureInfoURL) {
+        this.featureInfoURL = featureInfoURL;
     }
 
     public void setUseScore(boolean useScore) {
@@ -195,11 +193,6 @@ public abstract class AbstractTrack implements Track {
         this.sampleId = sampleId;
     }
 
-    @Override
-    public boolean isFilterable() {
-        return true;   // True by default
-    }
-
     public void renderName(Graphics2D g2D, Rectangle trackRectangle, Rectangle visibleRectangle) {
 
         Rectangle rect = getDisplayableRect(trackRectangle, visibleRectangle);
@@ -421,10 +414,6 @@ public abstract class AbstractTrack implements Track {
         this.minimumHeight = minimumHeight;
     }
 
-    public void setMaximumHeight(int maximumHeight) {
-        this.maximumHeight = maximumHeight;
-    }
-
     /**
      * Return the actual minimum height if one has been set, otherwise get the default for the current renderer.
      *
@@ -434,10 +423,6 @@ public abstract class AbstractTrack implements Track {
         return minimumHeight < 0 ? getDefaultMinimumHeight() : minimumHeight;
     }
 
-    public int getMaximumHeight() {
-        return maximumHeight;
-    }
-
     public void setTrackType(TrackType type) {
         this.trackType = type;
     }
@@ -505,7 +490,7 @@ public abstract class AbstractTrack implements Track {
         if (force) {
             this.height = preferredHeight;
         } else {
-            this.height = Math.min(Math.max(getMinimumHeight(), preferredHeight), getMaximumHeight());
+            this.height = Math.max(getMinimumHeight(), preferredHeight);
         }
     }
 
@@ -513,10 +498,6 @@ public abstract class AbstractTrack implements Track {
         return (height < 0) ? getDefaultHeight() : height;
     }
 
-    public boolean hasDataRange() {
-        return dataRange != null;
-    }
-
     public DataRange getDataRange() {
         if (dataRange == null) {
             // Use the color scale if there is one
@@ -689,7 +670,7 @@ public abstract class AbstractTrack implements Track {
             setWindowFunction(properties.getWindowingFunction());
         }
         if (properties.getUrl() != null) {
-            setUrl(properties.getUrl());
+            setFeatureInfoURL(properties.getUrl());
         }
 
         Map<String, String> attributes = properties.getAttributes();
