@@ -117,6 +117,7 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
     private String trackLine = null;
 
     private boolean groupByStrand = false;
+    private String labelField;
 
     public FeatureTrack() {
 
@@ -400,9 +401,7 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
                     buf.append(vs);
 
                     if (IGV.getInstance().isShowDetailsOnClick()) {
-                        // URL
                         String url = getFeatureURL(igvFeature);
-
                         if (url != null) {
                             buf.append("<br/><a href=\"" + url + "\">" + url + "</a>");
                         }
@@ -443,15 +442,24 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
     private String getFeatureURL(IGVFeature igvFeature) {
         String url = igvFeature.getURL();
         if (url == null) {
-            String trackURL = getUrl();
-            if (trackURL != null && igvFeature.getIdentifier() != null) {
-                String encodedID = StringUtils.encodeURL(igvFeature.getIdentifier());
+            String trackURL = getFeatureInfoURL();
+            if (trackURL != null && igvFeature.getName() != null) {
+                String name = labelField != null ? igvFeature.getDisplayName(labelField) : igvFeature.getName();
+                String encodedID = StringUtils.encodeURL(name);
                 url = trackURL.replaceAll("\\$\\$", encodedID);
             }
         }
         return url;
     }
 
+    @Override
+    public String getLabelField() {
+        return labelField;
+    }
+
+    public void setLabelField(String labelField) {
+        this.labelField = labelField;
+    }
 
     /**
      * Get all features which overlap the specified locus
@@ -463,7 +471,10 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
         try {
             Iterator<Feature> iter = source.getFeatures(chr, start, end);
             while (iter.hasNext()) {
-                features.add(iter.next());
+                Feature f = iter.next();
+                if(f.getEnd() >= start && f.getStart() <= end) {
+                    features.add(f);
+                }
             }
         } catch (IOException e) {
             throw new RuntimeException(e);
@@ -678,13 +689,22 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
 
         try {
 
-            int delta = (end - start) / 2;
-            int expandedStart = start - delta;
-            int expandedEnd = end + delta;
-            if (expandedEnd < 0) {
-                expandedEnd = Integer.MAX_VALUE;  // overflow
+            int expandedStart;
+            int expandedEnd;
+            int vw = getVisibilityWindow();
+            if (vw > 0) {
+                int delta = (end - start) / 2;
+                expandedStart = start - delta;
+                expandedEnd = end + delta;
+                if (expandedEnd < 0) {
+                    expandedEnd = Integer.MAX_VALUE;  // overflow
+                }
+            } else {
+                expandedStart = 0;
+                expandedEnd = Integer.MAX_VALUE;
             }
 
+
             //Make sure we are only querying within the chromosome we allow for somewhat pathological cases of start
             //being negative and end being outside, but only if directly queried. Our expansion should not
             //set start < 0 or end > chromosomeLength
@@ -855,7 +875,7 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
 
         Renderer renderer = getRenderer();
 
-        if (getDisplayMode() == DisplayMode.COLLAPSED) {
+        if (getDisplayMode() == DisplayMode.COLLAPSED && !isGroupByStrand()) {
             List<Feature> features = packedFeatures.getFeatures();
             if (features != null) {
                 renderer.render(features, context, inputRect, this);
@@ -903,20 +923,22 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
         boolean canScroll = (forward && !frame.windowAtEnd()) || (!forward && frame.getOrigin() > 0);
         PackedFeatures packedFeatures = packedFeaturesMap.get(frame.getName());
 
+        // Compute a buffer to define "next"
+        double buffer = Math.max(1, frame.getScale());
+
         if (packedFeatures != null && packedFeatures.containsInterval(chr, (int) center - 1, (int) center + 1)) {
             if (packedFeatures.getFeatures().size() > 0 && canScroll) {
+                List<Feature> centerSortedFeatures = packedFeatures.getCenterSortedFeatures();
                 f = (forward ?
-                        FeatureUtils.getFeatureAfter(center, packedFeatures.getFeatures()) :
-                        FeatureUtils.getFeatureBefore(center, packedFeatures.getFeatures()));
-            }
-            if (f == null) {
-                FeatureSource rawSource = source;
-                if (source instanceof CachingFeatureSource) {
-                    rawSource = ((CachingFeatureSource) source).getSource();
-                }
-                f = FeatureTrackUtils.nextFeature(rawSource, chr, packedFeatures.getStart(), packedFeatures.getEnd(), forward);
+                        FeatureUtils.getFeatureCenteredAfter(center + buffer, centerSortedFeatures) :
+                        FeatureUtils.getFeatureCenteredBefore(center - buffer, centerSortedFeatures));
             }
         }
+        if (f == null) {
+            int searchBuferSize = (int) (frame.getScale() * 1000);
+            f = FeatureTrackUtils.nextFeature(source, chr, packedFeatures.getStart(), packedFeatures.getEnd(), center, searchBuferSize, forward);
+        }
+
 
         return f;
     }
@@ -1022,7 +1044,9 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
     @Override
     public void marshalXML(Document document, Element element) {
         element.setAttribute("groupByStrand", String.valueOf(groupByStrand));
-
+        if (labelField != null) {
+            element.setAttribute("featureNameProperty", labelField);
+        }
         super.marshalXML(document, element);
 
     }
@@ -1032,6 +1056,10 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
 
         super.unmarshalXML(element, version);
 
+        if (element.hasAttribute("featureNameProperty")) {
+            this.labelField = element.getAttribute("featureNameProperty");
+        }
+
         this.groupByStrand = "true".equals(element.getAttribute("groupByStrand"));
 
         NodeList tmp = element.getElementsByTagName("SequenceMatchSource");
