@@ -31,7 +31,7 @@ import htsjdk.variant.vcf.VCFHeader;
 import org.broad.igv.bbfile.BBFileReader;
 import org.broad.igv.bedpe.BedPEParser;
 import org.broad.igv.bedpe.InteractionTrack;
-import org.broad.igv.bigwig.BigWigDataSource;
+import org.broad.igv.bbfile.BBDataSource;
 import org.broad.igv.blast.BlastMapping;
 import org.broad.igv.blast.BlastParser;
 import org.broad.igv.data.*;
@@ -247,7 +247,10 @@ public class TrackLoader {
 
                 for (Track track : newTracks) {
                     if (locator.getFeatureInfoURL() != null) {
-                        track.setUrl(locator.getFeatureInfoURL());
+                        track.setFeatureInfoURL(locator.getFeatureInfoURL());
+                    }
+                    if (locator.getLabelField() != null && track instanceof FeatureTrack) {
+                        ((FeatureTrack) track).setLabelField(locator.getLabelField());
                     }
                     if (tp != null) {
                         track.setProperties(tp);
@@ -312,10 +315,14 @@ public class TrackLoader {
     }
 
     public static boolean isAlignmentTrack(String typeString) {
-        return typeString.equals("sam") || typeString.equals("bam") || typeString.equals("cram") ||
-                typeString.equals("sam.list") || typeString.equals("bam.list") ||
-                typeString.equals("aligned") || typeString.equals("sai") ||
-                typeString.equals("bai") || typeString.equals("csi") || typeString.equals("alist");
+        if (typeString == null) {
+            return false;
+        } else {
+            return typeString.equals("sam") || typeString.equals("bam") || typeString.equals("cram") ||
+                    typeString.equals("sam.list") || typeString.equals("bam.list") ||
+                    typeString.equals("aligned") || typeString.equals("sai") ||
+                    typeString.equals("bai") || typeString.equals("csi") || typeString.equals("alist");
+        }
     }
 
     private void loadSMAPFile(ResourceLocator locator, List<Track> newTracks, Genome genome) throws IOException {
@@ -813,11 +820,11 @@ public class TrackLoader {
 
         String path = locator.getPath();
         BBFileReader reader = new BBFileReader(path);
-        BigWigDataSource bigwigSource = new BigWigDataSource(reader, genome);
+        BBDataSource bigwigSource = new BBDataSource(reader, genome);
+        Track track = null;
 
         if (reader.isBigWigFile()) {
-            DataSourceTrack track = new DataSourceTrack(locator, trackId, trackName, bigwigSource);
-            newTracks.add(track);
+            track = new DataSourceTrack(locator, trackId, trackName, bigwigSource);
         } else if (reader.isBigBedFile()) {
 
             if (locator.getPath().contains("RRBS_cpgMethylation") ||
@@ -825,12 +832,16 @@ public class TrackLoader {
                     (reader.getAutoSql() != null && reader.getAutoSql().startsWith("table BisulfiteSeq"))) {
                 loadMethylTrack(locator, reader, newTracks, genome);
             } else {
-                FeatureTrack track = new FeatureTrack(locator, trackId, trackName, bigwigSource);
-                newTracks.add(track);
+                track = new FeatureTrack(locator, trackId, trackName, bigwigSource);
             }
         } else {
             throw new RuntimeException("Unknown BIGWIG type: " + locator.getPath());
         }
+
+        if (track != null) {
+            newTracks.add(track);
+        }
+
     }
 
     private void loadMethylTrack(ResourceLocator locator, BBFileReader reader, List<Track> newTracks, Genome genome) throws IOException {
@@ -941,7 +952,7 @@ public class TrackLoader {
                                 path.contains("dataformat=.bam") ||
                                 path.contains("/query.cgi?");
                 if (!bypassFileAutoDiscovery) {
-                    covPath = path + ".tdf";
+                    covPath = ResourceLocator.appendToPath(locator, ".tdf");
                 }
             }
 
