@@ -37,6 +37,7 @@ import org.broad.igv.feature.Exon;
 import org.broad.igv.feature.Range;
 import org.broad.igv.feature.RegionOfInterest;
 import org.broad.igv.feature.genome.GenomeManager;
+import org.broad.igv.prefs.Constants;
 import org.broad.igv.prefs.IGVPreferences;
 import org.broad.igv.prefs.PreferencesManager;
 import org.broad.igv.sam.AlignmentTrack;
@@ -48,12 +49,16 @@ import org.broad.igv.ui.util.MessageUtils;
 import org.broad.igv.variant.VariantTrack;
 
 import javax.swing.*;
+import javax.swing.text.JTextComponent;
 import java.awt.*;
 import java.awt.event.ActionEvent;
 import java.awt.event.KeyEvent;
+import java.awt.event.KeyListener;
 import java.io.IOException;
 import java.util.Collection;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 
 import static org.broad.igv.prefs.Constants.*;
 
@@ -67,12 +72,29 @@ public class GlobalKeyDispatcher implements KeyEventDispatcher {
     private final InputMap inputMap = new InputMap();
     private final ActionMap actionMap = new ActionMap();
 
-    public GlobalKeyDispatcher() {
+    private static GlobalKeyDispatcher theInstance;
+
+    public static synchronized GlobalKeyDispatcher getInstance() {
+        if(theInstance == null) {
+            theInstance = new GlobalKeyDispatcher();
+        }
+        return theInstance;
+    }
+
+    private GlobalKeyDispatcher() {
         init();
     }
 
     public boolean dispatchKeyEvent(KeyEvent event) {
 
+        // If the source of this event is a text component don't process it here.
+        final Object source = event.getSource();
+        if(JTextComponent.class.isInstance(source) ||
+                TextComponent.class.isInstance(source)) {
+            return false;   // <= important, returning true will prevent further dispatching of event
+        }
+
+
         KeyStroke ks = KeyStroke.getKeyStrokeForEvent(event);
         String actionKey = (String) inputMap.get(ks);
 
@@ -94,7 +116,7 @@ public class GlobalKeyDispatcher implements KeyEventDispatcher {
             if (action != null && action.isEnabled()) {
                 // I'm not sure about the parameters
                 action.actionPerformed(
-                        new ActionEvent(event.getSource(), event.getID(),
+                        new ActionEvent(source, event.getID(),
                                 actionKey, ((KeyEvent) event).getModifiers()));
                 return true; // consume event
             }
@@ -411,15 +433,36 @@ public class GlobalKeyDispatcher implements KeyEventDispatcher {
 
                 if (f != null) {
                     String chr = GenomeManager.getInstance().getCurrentGenome().getCanonicalChrName(f.getChr());
-                    double newCenter = f.getStart();
-                    if (!chr.equals(frame.getChrName())) {
-                        // Switch chromosomes.  We have to do some tricks to maintain the same resolution scale.
-                        double range = frame.getEnd() - frame.getOrigin();
-                        int newOrigin = (int) Math.max(newCenter - range / 2, 0);
-                        int newEnd = (int) (newOrigin + range);
-                        frame.jumpTo(chr, newOrigin, newEnd);
-                    } else {
-                        frame.centerOnLocation(newCenter);
+                    double newCenter = (f.getStart() + f.getEnd()) / 2.0;
+
+                    boolean fitToWindow = PreferencesManager.getPreferences().getAsBoolean(NEXT_FIT_TO_WINDOW);
+
+                    if(fitToWindow) {
+                        int flankingRegion = PreferencesManager.getPreferences().getAsInt(Constants.NEXT_FLANKING_REGION);
+                        int delta;
+                        int start = f.getStart();
+                        int end = f.getEnd();
+                        if ((end - start) == 1) {
+                            delta = 20; // Don't show flanking region for single base jumps, use 40bp window
+                        } else if (flankingRegion < 0) {
+                            delta = (-flankingRegion * (end - start)) / 100;
+                        } else {
+                            delta = flankingRegion;
+                        }
+                        start = Math.max(0, start - delta);
+                        end = end + delta;
+                        frame.jumpTo(chr, start, end);
+                    }
+                    else {
+                        if (!chr.equals(frame.getChrName())) {
+                            // Switch chromosomes.  We have to do some tricks to maintain the same resolution scale.
+                            double range = frame.getEnd() - frame.getOrigin();
+                            int newOrigin = (int) Math.max(newCenter - range / 2, 0);
+                            int newEnd = (int) (newOrigin + range);
+                            frame.jumpTo(chr, newOrigin, newEnd);
+                        } else {
+                            frame.centerOnLocation(newCenter);
+                        }
                     }
                 }
             } catch (IOException e) {
