@@ -993,11 +993,6 @@ public class IGV implements IGVEventObserver {
                 //If there's a RegionNavigatorDialog, kill it.
                 //this could be done through the Observer that RND uses, I suppose.  Not sure that's cleaner
                 RegionNavigatorDialog.destroyInstance();
-
-                if (PreferencesManager.getPreferences().getAsBoolean(Constants.CIRC_VIEW_ENABLED) && CircularViewUtilities.ping()) {
-                    CircularViewUtilities.clearAll();
-                }
-
             }
 
             return success;
@@ -1292,19 +1287,29 @@ public class IGV implements IGVEventObserver {
         }
     }
 
+    /**
+     * Return a DataPanel for the given track.
+     *
+     * @param track
+     * @return
+     */
     public TrackPanel getPanelFor(Track track) {
-        ResourceLocator locator = track.getResourceLocator();
-        String path = locator.getPath().toLowerCase();
-        if ("alist".equals(locator.getFormat())) {
-            return getVcfBamPanel();
-        } else if (PreferencesManager.getPreferences().getAsBoolean(SHOW_SINGLE_TRACK_PANE_KEY)) {
+
+        if (PreferencesManager.getPreferences().getAsBoolean(SHOW_SINGLE_TRACK_PANE_KEY)) {
             return getTrackPanel(DATA_PANEL_NAME);
-        } else if (TrackLoader.isAlignmentTrack(locator.getFormat())) {
-            String newPanelName = "Panel" + System.currentTimeMillis();
-            return addDataPanel(newPanelName).getTrackPanel();
-        } else if (track instanceof FeatureTrack) {
+        }
+
+        ResourceLocator locator = track.getResourceLocator();
+        if (locator != null) {
+            TrackPanel panel = getPanelFor(locator);
+            if (panel != null) {
+                return panel;
+            }
+        }
+
+        if(track.getClass() == FeatureTrack.class && !PreferencesManager.getPreferences().getAsBoolean(SHOW_SINGLE_TRACK_PANE_KEY))
             return getTrackPanel(FEATURE_PANEL_NAME);
-        } else {
+        else {
             return getTrackPanel(DATA_PANEL_NAME);
         }
     }
@@ -1318,15 +1323,23 @@ public class IGV implements IGVEventObserver {
      */
     public TrackPanel getPanelFor(ResourceLocator locator) {
 
-        if ("alist".equals(locator.getFormat())) {
+        final String format = locator.getFormat();
+        if ("alist".equals(format)) {
             return getVcfBamPanel();
         } else if (PreferencesManager.getPreferences().getAsBoolean(SHOW_SINGLE_TRACK_PANE_KEY)) {
             return getTrackPanel(DATA_PANEL_NAME);
-        } else if (TrackLoader.isAlignmentTrack(locator.getFormat())) {
+        } else if (TrackLoader.isAlignmentTrack(format)) {
             String newPanelName = "Panel" + System.currentTimeMillis();
             return addDataPanel(newPanelName).getTrackPanel();
         } else {
-            return getDefaultPanel(locator);
+            if (format != null && format.equalsIgnoreCase("das")) {
+                return getTrackPanel(FEATURE_PANEL_NAME);
+            }
+            if (isAnnotationFile(format)) {
+                return getTrackPanel(FEATURE_PANEL_NAME);
+            } else {
+                return null;  // Can't determine from locator
+            }
         }
     }
 
@@ -1357,24 +1370,10 @@ public class IGV implements IGVEventObserver {
         }
     }
 
-
-    private TrackPanel getDefaultPanel(ResourceLocator locator) {
-
-        final String format = locator.getFormat();
-        if (format != null && format.equalsIgnoreCase("das")) {
-            return getTrackPanel(FEATURE_PANEL_NAME);
-        }
-        if (isAnnotationFile(locator.getFormat())) {
-            return getTrackPanel(FEATURE_PANEL_NAME);
-        } else {
-            return getTrackPanel(DATA_PANEL_NAME);
-        }
-    }
-
     private boolean isAnnotationFile(String format) {
         Set<String> annotationFormats = new HashSet<>(Arrays.asList("refflat", "ucscgene",
                 "genepred", "ensgene", "refgene", "gff", "gtf", "gff3", "embl", "bed", "gistic",
-                "bedz", "repmask", "dranger", "ucscsnp", "genepredext"));
+                "bedz", "repmask", "dranger", "ucscsnp", "genepredext", "bigbed"));
         return annotationFormats.contains(format);
     }
 
