@@ -25,7 +25,6 @@
 
 package org.broad.igv.ui;
 
-import org.broad.igv.logging.*;
 import org.broad.igv.DirectoryManager;
 import org.broad.igv.Globals;
 import org.broad.igv.annotations.ForTesting;
@@ -41,11 +40,11 @@ import org.broad.igv.google.GoogleUtils;
 import org.broad.igv.google.OAuthProvider;
 import org.broad.igv.google.OAuthUtils;
 import org.broad.igv.lists.GeneListManagerUI;
+import org.broad.igv.logging.LogManager;
+import org.broad.igv.logging.Logger;
 import org.broad.igv.prefs.PreferencesManager;
-import org.broad.igv.session.Session;
 import org.broad.igv.tools.IgvToolsGui;
 import org.broad.igv.tools.motiffinder.MotifFinderPlugin;
-import org.broad.igv.track.AttributeManager;
 import org.broad.igv.track.CombinedDataSourceDialog;
 import org.broad.igv.ui.action.*;
 import org.broad.igv.ui.commandbar.GenomeComboBox;
@@ -193,7 +192,7 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
 
         try {
             AWSMenu = createAWSMenu();
-            AWSMenu.setVisible(AmazonUtils.isAWSProviderPresent());
+            AWSMenu.setVisible(AmazonUtils.isAwsProviderPresent());
             menus.add(AWSMenu);
 
         } catch (IOException e) {
@@ -209,7 +208,7 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
     }
 
     public void updateAWSMenu() {
-        AWSMenu.setVisible(AmazonUtils.isAWSProviderPresent());
+        AWSMenu.setVisible(AmazonUtils.isAwsProviderPresent());
     }
 
     /**
@@ -788,56 +787,6 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         return MenuAndToolbarUtils.createMenu(menuItems, helpMenuAction);
     }
 
-    private void checkVersion() {
-
-        int readTimeout = Globals.READ_TIMEOUT;
-        int connectTimeout = Globals.CONNECT_TIMEOUT;
-
-        try {
-            Main.Version thisVersion = Main.Version.getVersion(Globals.VERSION);
-
-            if (thisVersion != null) {
-
-                Globals.CONNECT_TIMEOUT = 5000;
-                Globals.READ_TIMEOUT = 1000;
-                final String serverVersionString = HttpUtils.getInstance().getContentsAsString(HttpUtils.createURL(Globals.getVersionURL())).trim();
-                // See if user has specified to skip this update
-
-                final String skipString = PreferencesManager.getPreferences().get(SKIP_VERSION);
-                HashSet<String> skipVersion = new HashSet<String>(Arrays.asList(skipString.split(",")));
-                if (skipVersion.contains(serverVersionString)) return;
-
-                Main.Version serverVersion = Main.Version.getVersion(serverVersionString.trim());
-                if (serverVersion == null) return;
-
-                if (thisVersion.lessThan(serverVersion)) {
-
-                    log.info("A later version of IGV is available (" + serverVersionString + ")");
-                    final VersionUpdateDialog dlg = new VersionUpdateDialog(serverVersionString);
-
-                    dlg.setVisible(true);
-                    if (dlg.isSkipVersion()) {
-                        String newSkipString = skipString + "," + serverVersionString;
-                        PreferencesManager.getPreferences().put(SKIP_VERSION, newSkipString);
-                    }
-
-                } else {
-                    MessageUtils.showMessage("IGV is up to date");
-                }
-            } else {
-                if (Globals.VERSION.contains("3.0_beta") || Globals.VERSION.contains("snapshot")) {
-                    HttpUtils.getInstance().getContentsAsString(HttpUtils.createURL(Globals.getVersionURL())).trim();
-                }
-            }
-
-        } catch (Exception e) {
-            log.error("Error checking version", e);
-        } finally {
-            Globals.CONNECT_TIMEOUT = connectTimeout;
-            Globals.READ_TIMEOUT = readTimeout;
-        }
-    }
-
     private JMenu createExtrasMenu() {
 
         List<JComponent> menuItems = new ArrayList<JComponent>();
@@ -974,29 +923,36 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
 
     private JMenu createAWSMenu() throws IOException {
 
+        boolean usingCognito = AmazonUtils.GetCognitoConfig() != null;
+
         JMenu menu = new JMenu("Amazon");
 
+        // Login
         final JMenuItem login = new JMenuItem("Login");
         login.addActionListener(e -> {
             try {
                 OAuthProvider oauth = OAuthUtils.getInstance().getProvider("Amazon");
-                oauth.openAuthorizationPage(); // should trigger and event and UI takes over
+                oauth.openAuthorizationPage();
             } catch (Exception ex) {
                 MessageUtils.showErrorMessage("Error fetching oAuth tokens.  See log for details", ex);
                 log.error("Error fetching oAuth tokens", ex);
             }
         });
-        login.setEnabled(false);
+        login.setEnabled(usingCognito);
+        login.setVisible(usingCognito);
         menu.add(login);
 
+        // Logout
         final JMenuItem logout = new JMenuItem("Logout");
         logout.addActionListener(e -> {
             OAuthProvider oauth = OAuthUtils.getInstance().getProvider("Amazon");
             oauth.logout();
         });
         logout.setEnabled(false);
+        logout.setVisible(usingCognito);
         menu.add(logout);
 
+        // Load item, added to menu later
         final JMenuItem loadS3 = new JMenuItem("Load from S3 bucket");
         loadS3.addActionListener(e -> {
             List<String> buckets = AmazonUtils.ListBucketsForUser();
@@ -1009,43 +965,44 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
                 dlg.dispose();
             });
         });
-        loadS3.setEnabled(false);
+        loadS3.setEnabled(!usingCognito);  // If using Cognito, disalbe initially
         menu.add(loadS3);
 
         menu.addMenuListener(new MenuListener() {
             @Override
             public void menuSelected(MenuEvent e) {
-                Runnable runnable = () -> {
-                    OAuthProvider oauth = OAuthUtils.getInstance().getProvider("Amazon");
-                    boolean loggedIn = false;
-                    loggedIn = oauth.isLoggedIn();
-                    log.debug("MenuBar is user loggedIn?: " + loggedIn);
+                if (AmazonUtils.GetCognitoConfig() != null) {
+                    Runnable runnable = () -> {
+                        OAuthProvider oauth = OAuthUtils.getInstance().getProvider("Amazon");
+                        boolean loggedIn = false;
+                        loggedIn = oauth.isLoggedIn();
+                        log.debug("MenuBar is user loggedIn?: " + loggedIn);
 
-                    if (loggedIn) {
-                        login.setText(oauth.getCurrentUserName());
-                    } else {
-                        login.setText("Login ...");
-                    }
-                    login.setEnabled(!loggedIn);
-                    logout.setEnabled(loggedIn);
-                    loadS3.setEnabled(loggedIn);
-                };
-
-                LongRunningTask.submit(runnable);
+                        if (loggedIn) {
+                            login.setText(oauth.getCurrentUserName());
+                        } else {
+                            login.setText("Login ...");
+                        }
+                        login.setVisible(true);
+                        logout.setVisible(true);
+                        login.setEnabled(!loggedIn);
+                        logout.setEnabled(loggedIn);
+                        loadS3.setEnabled(loggedIn);
+                    };
+                    LongRunningTask.submit(runnable);
+                }
             }
 
             @Override
             public void menuDeselected(MenuEvent e) {
-
             }
 
             @Override
             public void menuCanceled(MenuEvent e) {
-
             }
-
         });
 
+
         return menu;
     }
 
