@@ -27,26 +27,21 @@ package org.broad.igv.util;
 
 import com.google.gson.JsonObject;
 import htsjdk.tribble.Tribble;
-import org.broad.igv.logging.*;
-import org.broad.igv.data.cufflinks.FPKMTrackingCodec;
-import org.broad.igv.feature.FeatureType;
-import org.broad.igv.feature.dsi.DSICodec;
-import org.broad.igv.feature.tribble.IntervalListCodec;
-import org.broad.igv.feature.tribble.MUTCodec;
-import org.broad.igv.feature.tribble.PAFCodec;
-import org.broad.igv.feature.tribble.UCSCGeneTableCodec;
 import org.broad.igv.google.GoogleUtils;
+import org.broad.igv.logging.LogManager;
+import org.broad.igv.logging.Logger;
 
-//import java.awt.*;
 import java.awt.*;
 import java.io.File;
 import java.net.MalformedURLException;
 import java.net.URL;
-import java.util.*;
 import java.util.List;
+import java.util.*;
 
 import static org.broad.igv.feature.tribble.CodecFactory.ucscSNP;
 
+//import java.awt.*;
+
 /**
  * Represents a data file or other resource, which might be local file or remote resource.
  *
@@ -98,6 +93,8 @@ public class ResourceLocator {
      */
     String featureInfoURL;
 
+    String labelField;
+
     /**
      * Descriptive text
      */
@@ -244,7 +241,7 @@ public class ResourceLocator {
         this.visibilityWindow = visibilityWindow;
     }
 
-    private String deriveFormat(String pathOrName) {
+    public static String deriveFormat(String pathOrName) {
 
         String filename;
         if (FileUtils.isRemote(pathOrName)) {
@@ -285,20 +282,11 @@ public class ResourceLocator {
             return "cds_exp.diff";
         } else if (filename.endsWith("genepredext")) {
             return "genepredext";
-        } else if (filename.contains("refflat")) {
-            return "reflat";
-        } else if (filename.contains("genepred") || filename.contains("ensgene") ||
-                filename.contains("refgene") || filename.contains("ncbirefseq")) {
-            return "refgene";
-        } else if (filename.contains("ucscgene")) {
-            return "ucscgene";
         } else if (filename.endsWith(".maf.annotated")) {
             // TCGA extension
             return "mut";
         } else if (filename.endsWith("junctions.bed")) {
             return "junctions";
-        } else if (filename.matches(ucscSNP)) {
-            return "snp";
         } else if (filename.endsWith("bam.list")) {
             return "bam.list";
         } else if (filename.endsWith("sam.list")) {
@@ -314,8 +302,21 @@ public class ResourceLocator {
         } else if (filename.endsWith("_clusters")) {
             return "bedpe";
         } else {
-            // Default - return the extension
-            return filename.substring(filename.lastIndexOf('.') + 1).toLowerCase();
+            // Default - derive format from the extension.  If not a common format check special cases
+            String format = filename.substring(filename.lastIndexOf('.') + 1).toLowerCase();
+            if (!knownFormats.contains(format)) {
+                if (filename.contains("refflat")) {
+                    return "reflat";
+                } else if (filename.contains("genepred") || filename.contains("ensgene") ||
+                        filename.contains("refgene") || filename.contains("ncbirefseq")) {
+                    return "refgene";
+                } else if (filename.contains("ucscgene")) {
+                    return "ucscgene";
+                } else if (filename.matches(ucscSNP)) {
+                    return "snp";
+                }
+            }
+            return format;
         }
     }
 
@@ -375,7 +376,7 @@ public class ResourceLocator {
         return dbURL == null && !dataURL && !FileUtils.isRemote(path);
     }
 
-    public void setTrackInforURL(String trackInforURL) {
+    public void setTrackInfoURL(String trackInforURL) {
         this.trackInforURL = trackInforURL;
     }
 
@@ -429,6 +430,14 @@ public class ResourceLocator {
         return featureInfoURL;
     }
 
+    public String getLabelField() {
+        return labelField;
+    }
+
+    public void setLabelField(String labelField) {
+        this.labelField = labelField;
+    }
+
     public void setFeatureInfoURL(String featureInfoURL) {
         this.featureInfoURL = featureInfoURL;
     }
@@ -522,20 +531,20 @@ public class ResourceLocator {
 
 
     /**
-     * Add the {@code indexExtension} to the path in locator, preserving
+     * Add the {@code extension} to the path in locator, preserving
      * query string elements if present
      *
      * @param locator
-     * @param indexExtension
+     * @param extension
      * @return
      */
-    public static String appendToPath(ResourceLocator locator, String indexExtension) {
-        String indexFile = locator.getURLPath() + indexExtension;
+    public static String appendToPath(ResourceLocator locator, String extension) {
+        String extendedPath = locator.getURLPath() + extension;
         String qs = locator.getURLQueryString();
         if (qs != null && qs.length() > 0) {
-            indexFile += "?" + qs;
+            extendedPath += "?" + qs;
         }
-        return indexFile;
+        return extendedPath;
     }
 
     /**
@@ -558,14 +567,6 @@ public class ResourceLocator {
         }
     }
 
-    public void setAttribute(String key, Object value) {
-        this.attributes.put(key, value);
-    }
-
-    public Object getAttribute(String key) {
-        return attributes.get(key);
-    }
-
     public void setIndexed(boolean indexed) {
         this.indexed = indexed;
     }
@@ -613,6 +614,7 @@ public class ResourceLocator {
         SAMPLE_ID("sampleId"),
         NAME("name"),
         URL("url"),
+        LABEL_FIELD("labelField"),
         RESOURCE_TYPE("resourceType"),
         TRACK_LINE("trackLine"),
         COVERAGE("coverage"),
@@ -637,4 +639,7 @@ public class ResourceLocator {
         }
 
     }
+
+    static Set<String> knownFormats = new HashSet<>(Arrays.asList("gff", "bed", "gtf", "gff3",
+            "seg", "bb", "bigbed", "bigwig", "bam", "cram", "vcf"));
 }
