@@ -74,7 +74,7 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
 
     static final DecimalFormat numFormat = new DecimalFormat("#.###");
 
-    private static final Color CIRC_VIEW_DEFAULT_COLOR = new Color(27,192,249);
+    private static final Color CIRC_VIEW_DEFAULT_COLOR = new Color(27, 192, 249);
     private static final int GROUP_BORDER_WIDTH = 3;
     private static final Color BAND1_COLOR = new Color(245, 245, 245);
     private static final Color BAND2_COLOR = Color.white;
@@ -113,7 +113,7 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
      */
     private int top;
 
-    private boolean showGenotypes = true;
+    private boolean showGenotypes;
 
     /**
      * The height of a single row in in squished mode
@@ -228,9 +228,8 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
         final int margins = (groupCount - 1) * 3;
         squishedHeight = sampleCount == 0 || showGenotypes == false ? DEFAULT_SQUISHED_HEIGHT :
                 Math.min(DEFAULT_SQUISHED_HEIGHT, Math.max(1, (height - getVariantBandHeight() - margins) / sampleCount));
-        if (sampleCount == 1) {
-            showGenotypes = false;
-        }
+        showGenotypes = defaultShowGenotypes();
+
 
         // If sample->bam list file is supplied enable vcfToBamMode.
         String vcfToBamMapping = locator == null ? null : locator.getMappingPath();
@@ -263,6 +262,10 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
 
     }
 
+    private boolean defaultShowGenotypes() {
+        return allSamples.size() > 0;
+    }
+
     private void loadAlignmentMappings(String bamListPath) {
         alignmentFiles = new HashMap<String, String>();
         BufferedReader br = null;
@@ -1562,8 +1565,7 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
 
         super.marshalXML(document, element);
 
-        boolean defaultShowGenotypes = allSamples.size() == 1 ? false : true;
-        if (showGenotypes != defaultShowGenotypes) {
+        if (showGenotypes != defaultShowGenotypes()) {
             element.setAttribute("showGenotypes", String.valueOf(showGenotypes));
         }
 
@@ -1587,7 +1589,7 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
         super.unmarshalXML(element, version);
 
         if (element.hasAttribute("showGenotypes")) {
-            this.showGenotypes = Boolean.getBoolean(element.getAttribute("showGenotypes"));
+            this.showGenotypes = Boolean.parseBoolean(element.getAttribute("showGenotypes"));
         }
 
         if (element.hasAttribute("squishedHeight")) {
